/*

~~~~~~~~~~~~~~~~
~ main-gui.cpp ~
~~~~~~~~~~~~~~~~

Dies ist das Hauptprogramm der GUI-Komponente von Eukalyptos. Hier wird das GUI
erstellt und fr die Verwendung vom Benutzer vorbereitet.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/


#include <Fl/Fl.h>
#include <FL/fl_ask.h>
#include <Fl/Fl_Window.h>

#include "module_map.h"
#include "module_net_tcp.h"
#include "module_conf_ConfigFile.h"
#include "gui.h"

int main( void )
{
	srand( time( NULL ) );

	// Hier werden die Module erklrt und beim module_master registriert. Die Reihenfolge
	// der Registrierungen ist wichtig, da manche Module voneinenader Abhngen. Grundstzlich
	// sollte das Konfigurationsmodul als erstes Modul registriert werden, denn es bietet
	// den anderen Modulen die Mglichkeit, Konfigurationsdaten auszulesen.
	module_net_tcp client;
	module_conf_ConfigFile conf;
	module_map map;
	module_log log;

	module_master::ptr().register_module( &conf );
	module_master::ptr().register_module( &log );	
	module_master::ptr().register_module( &client );
	module_master::ptr().register_module( &map );

	double KARTENBREITE = atof( conf.da_datum( "KARTENBREITE" ) );
	double KARTENHOEHE = atof( conf.da_datum( "KARTENHOEHE" ) );

	map.setup( KARTENBREITE, KARTENHOEHE );

	
	// VORSICHT!
	//
	// Der Parser muss _nach_ dem Configfile geladen bzw.
	// registriert werden!
	module_parser parser;
	module_master::ptr().register_module( &parser );

	GUI_Window GUI( 680, 640, "Eukalyptos" );
	GUI.show();

	Fl::run();
	
	module_master::ptr().unload_all();
	return( 0 );
}
