/*

~~~~~~~~~~~~~~~~~~~
~ main-server.cpp ~
~~~~~~~~~~~~~~~~~~~

Dies ist die Serverkomponente des Eukalyptos-Programms. Hier werden die Befehle
vom GUI entgegegenommen und an den Roboter gesendet. Ebenso findet eine
berprfung der Befehlsbertragung statt.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/

// TODO:
// - Fehlerhaftes Datalog anders formatieren
// - check_command_datalog kommentieren

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <math.h>

#include <unistd.h>
#include <sys/time.h>

#include <sstream>
#include <vector>

using namespace std;

#include "protocol.h"
#include "module_net_tcp.h"
#include "module_conf_ConfigFile.h"
#include "module_robocom_linuxnqc.h"
#include "module_comp_linuxnqc.h"

const unsigned int SLEEP_TIME = 1;

bool check_command_datalog( vector<saved_net_msg> received_messages, string datalog, module_conf* mod_conf, vector<string>* events, int* lost_cmd, int* lost_param, bool* cmds_cut)
{
	unsigned char MSG_START_PHASE_1 = static_cast<unsigned char>( atoi( mod_conf->da_datum( "START_PHASE_1" ) ) );
	unsigned char MSG_START_PHASE_2 = static_cast<unsigned char>( atoi( mod_conf->da_datum( "START_PHASE_2" ) ) );
	unsigned char MSG_PREPARE = static_cast<unsigned char>( atoi( mod_conf->da_datum( "ROBOTERBEFEHL" ) ) );
	unsigned char MSG_HALT = static_cast<unsigned char>( atoi( mod_conf->da_datum( "ABBRUCH" ) ) );
	unsigned char MSG_CODE = static_cast<unsigned char>( atoi( mod_conf->da_datum( "MASZGESCHNEIDERTER_BEFEHL" ) ) );

	unsigned int EMPFANGSLISTE_ANFANG = static_cast<unsigned int>( atoi( mod_conf->da_datum( "EMPFANGSLISTE_ANFANG" ) ) );
	unsigned int FATALER_FEHLER = static_cast<unsigned int>( atoi( mod_conf->da_datum( "FATALER_FEHLER" ) ) );
	
	unsigned int SENSOR_1_AN = static_cast<unsigned int>( atoi( mod_conf->da_datum( "SENSOR_1_AN" ) ) );
	unsigned int SENSOR_2_AN = static_cast<unsigned int>( atoi( mod_conf->da_datum( "SENSOR_2_AN" ) ) );
	unsigned int SENSOR_3_AN = static_cast<unsigned int>( atoi( mod_conf->da_datum( "SENSOR_3_AN" ) ) );
	unsigned int SENSOR_1_AUS = static_cast<unsigned int>( atoi( mod_conf->da_datum( "SENSOR_1_AUS" ) ) );
	unsigned int SENSOR_2_AUS = static_cast<unsigned int>( atoi( mod_conf->da_datum( "SENSOR_2_AUS" ) ) );
	unsigned int SENSOR_3_AUS = static_cast<unsigned int>( atoi( mod_conf->da_datum( "SENSOR_3_AUS" ) ) );


	string temp;
	unsigned int code = 0;

	ostringstream conv;

	fprintf( stderr, 	"*** DATALOG MIT EMPFANGENEN BEFEHLEN ***\n"
				"%s\n"
				"****************************************\n\n", datalog.c_str() );
	
	// Zunchst wird berprft, ob es sich berhaupt um ein gltiges Datalog
	// handelt.
	temp = datalog.substr( 0, datalog.find_first_of( "\n" ) );
	if( atoi( temp.c_str() ) != EMPFANGSLISTE_ANFANG )
	{
		fprintf( stderr, "EMPFANGSLISTE_ANFANG\n" );
		module_master::ptr().broadcast( "#Server: Der Roboter hat nicht alle Befehle empfangen.\n" );
		return( false );
	}
	datalog.erase( 0, datalog.find_first_of( "\n" ) + 1 );

	int cur_cmd = 0;
	int cur_param = 0;
	int max_cmd = received_messages.size();
	
	bool waiting_for_params = false;
	bool error = false;

	while( datalog.find_first_of( "\n" ) != string::npos && cur_cmd < max_cmd )
	{
		temp = datalog.substr( 0, datalog.find_first_of( "\n" ) );
		code = static_cast<unsigned int>( atoi( temp.c_str() ) );

		fprintf( stderr, "temp = *%s*\n", temp.c_str() ); 

		// Ein Fehler oder ein anderes Ereignis ist aufgetreten.
		if( code > 255 )
		{
			if( code == FATALER_FEHLER )
			{
				conv << FATALER_FEHLER;
				
				temp =  conv.str() + "\n";
				temp += datalog.substr( temp.length() - 1, datalog.find_first_of( conv.str() + "\n" ) );
				
				// Achtung! Hier mssen die anschlieenden Vorkommen von FATALER_FEHLER gelscht werden,
		            	// sonst wird etwa so etwas in events gespeichert werden, falls der Datalog 666 666 666 666 enthlt:
            			// 666 666 666 666
            			// 666 666 666
            			// 666 666
            			// 666
            
				conv.flush();
				conv.clear();
					
				events->push_back( temp );
			}
			else if( 	code == SENSOR_1_AN ||
					code == SENSOR_2_AN ||
					code == SENSOR_3_AN ||
					code == SENSOR_1_AUS ||
					code == SENSOR_2_AUS ||
					code == SENSOR_3_AUS )

			{
				conv << code;
				temp = conv.str() + "\n";

				datalog.erase( 0, datalog.find_first_of( "\n" ) +1 );
				temp += datalog.substr( 0, datalog.find_first_of( "\n" ) );
			
				events->push_back( temp );
			}
		}

		// Der Befehl oder der Parameter wurde empfangen. Nun ist zu 
		// berprfen, ob der Empfang korrekt war.
		else
		{
			// Ein komplett neuer Header kann gelesen werden. Eventuell wartet
			// die Funktion danach auf neue Parameter.
			if( !waiting_for_params )
			{
				// Spezialkommandos ignorieren
				if( 	received_messages[ cur_cmd ].header.get_cmd() == MSG_START_PHASE_1 ||
					received_messages[ cur_cmd ].header.get_cmd() == MSG_START_PHASE_2 ||
					received_messages[ cur_cmd ].header.get_cmd() == MSG_PREPARE )
				{
					cur_cmd++;
					continue;
				}

				// Ha ha! Ein Fehler!
				if( received_messages[ cur_cmd ].header.get_cmd() != static_cast<unsigned char>( code ) )
				{
					error = true;
					break;
				}

				if( received_messages[ cur_cmd ].header.get_size() > 0 )
					waiting_for_params = true;
				else
					cur_cmd++; // Zum nchsten Kommando	

			}
		
			else
			{
				if( received_messages[ cur_cmd ].params[ cur_param ] != static_cast<unsigned char>( code ) )
				{
					error = true;
					break;
				}
				
				cur_param++;

				// Nicht zu viele Parameter lesen
				if( cur_param >= received_messages[ cur_cmd ].header.get_size() )
				{
					cur_param = 0;
					cur_cmd++;
					waiting_for_params = false;
				}
			}
		}

		datalog.erase( 0, datalog.find_first_of( "\n" ) + 1 );
      
		//Optimierung Anfang
      		if ( (datalog.find_first_of( "\n" ) == string::npos) && (cur_cmd==max_cmd-1) && (!waiting_for_params))
		{
			cur_param=-1; 
			waiting_for_params = true;
      		}
		//Optimierung Ende
	}
   
	//Optimierung Anfang   
	if ( (cur_cmd < max_cmd - 1) && (!error) )
	{
		*cmds_cut = true;
      		if ( waiting_for_params )
			*lost_param = cur_param; 
      		else
      			*lost_param = -1;
      
      		*lost_cmd = cur_cmd;
   	}
   
	if ( (cur_cmd==max_cmd-1) && (waiting_for_params) && (!error) && (received_messages[ cur_cmd ].header.get_cmd() != MSG_CODE) )
	{
		*cmds_cut = true;
		*lost_cmd = max_cmd-1;
		*lost_param = cur_param;
	}
   	//Optimierung Ende
   
   	// Das sollten alle Fehlerbedingungen sein...
	//
	// TODO:
	// Hier steckt noch ein Bug drin!
   	if ( error || *cmds_cut )
	{
      		module_master::ptr().broadcast( "!Server: Es konnten nicht alle Befehle erfolgreich an den Roboter gesendet werden.\n" );
		return( false );
	}
	else
		return( true );   
}

/// Liefert die Differenz zweier Zeitobjekte in Millisekunden. Die Funktion difftime aus der
/// Standardbibliothek kann nur eine Differenz in Sekunden berechnen.
///
/// @param time1 Erstes Ergebnis der Zeitmessung
/// @param time2 Zweites Ergebnis der Zeitmessung
///
/// @return Die Zeitdifferenz in Millisekunden als unsigned long
unsigned long difftime_msec( timeval time1, timeval time2 )
{
	return(   time2.tv_sec * 1000 + time2.tv_usec / 1000
		- time1.tv_sec * 1000 - time1.tv_usec / 1000 );
}

/// Hauptfunktion des Serverprogramms. Von hier aus werden, sobald neue Nachrichten auf der
/// Netzwerkschnittstelle anliegen, die entsprechenden Untermodule aufgerufen. Nach dem
/// Abarbeiten aller Befehle geht das Programm wieder in einen Wartezustand ber.
int main( void )
{
	module_net_tcp mod_tcp;
	module_conf_ConfigFile mod_conf;
	module_comp_linuxnqc mod_comp;
	module_robocom_linuxnqc mod_robocom;

	// Module registrieren. Die Reihenfolge der Registrierung ist
	// wichtig.

	module_master::ptr().register_module( &mod_tcp );
	module_master::ptr().register_module( &mod_conf );
	module_master::ptr().register_module( reinterpret_cast<module*>( &mod_comp ) );
	module_master::ptr().register_module( reinterpret_cast<module*>( &mod_robocom ) ); 

	// (Typen)daten aus der Konfigurationsdatei holen

	unsigned char MSG_START_PHASE_1 = static_cast<unsigned char>( atoi( mod_conf.da_datum( "START_PHASE_1" ) ) );
	unsigned char MSG_START_PHASE_2 = static_cast<unsigned char>( atoi( mod_conf.da_datum( "START_PHASE_2" ) ) );
	unsigned char MSG_PREPARE = static_cast<unsigned char>( atoi( mod_conf.da_datum( "ROBOTERBEFEHL" ) ) ); 
	unsigned char MSG_HALT = static_cast<unsigned char>( atoi( mod_conf.da_datum( "ABBRUCH" ) ) );
	unsigned char MSG_CODE  = static_cast<unsigned char>( atoi( mod_conf.da_datum( "MASZGESCHNEIDERTER_BEFEHL" ) ) );
	unsigned int EUKALYPTOS_PORT = static_cast<unsigned int>( atoi( mod_conf.da_datum( "PORT" ) ) );
	unsigned int SENDEVERSUCHE = static_cast<unsigned int>( atoi( mod_conf.da_datum( "SENDEVERSUCHE" ) ) );

	mod_tcp.connect( EUKALYPTOS_PORT );

	// Wie lange der Server schlafen soll (in Sekunden). Wird durch MSG_EXECUTION_TIME
	// ber das GUI gesetzt.
	unsigned int sleep_time = 0;
	
	// Hlt fest, an welchem Punkt der Konversation der Server mit dem
	// Roboter angelangt ist.
	unsigned short conversation_status = CONVERSATION_READ_ARCHIVED_COMMANDS;

	int bytes = 0;		// Anzahl Bytes, die auf der Netzwerschnittstelle zur Verfgung stehen.
	char* buf = NULL;	// Puffer zum Speichern der Nachricht. Wird immer auf die aktuelle
				// Gre der Netzwerkqueue gesetzt.
	bool reanimate = false;	// Flag, das gesetzt wird, wenn der Roboter reanimiert werden muss. Dies ist beispielsweise
				// nach Erhalt der "Schneiderlein"-Befehle der Fall.

	ostringstream conv;	// Zur Konvertierung von int->string			
	
	net_msg_hdr hdr;
	net_msg_hdr ping( MSG_TYPE_SYSTEM, MSG_PING );

	vector<saved_net_msg> received_messages;
	vector<string> events;
	string datalog;

	// Vorbereitung fr die Zeitmessung. Siehe unten.
	timeval time_now;
	timeval time_tmp;

	gettimeofday( &time_now, NULL );
	while( ( bytes = mod_tcp.has_msg() ) >= 0 )
	{
		gettimeofday( &time_tmp, NULL );

		// Der Server ruft erst dann eine Nachricht ab, wenn er mindestens
		// einen Header aus dem Datenstrom lesen kann.
		if( bytes >= sizeof( hdr ) )
		{
			buf = new char[ sizeof( hdr ) + 1];
			memset( buf, 0, sizeof( hdr ) + 1);

			mod_tcp.recv_msg( buf, sizeof( hdr ) );
			memcpy( &hdr, buf, sizeof( hdr ) );

			delete[] buf;

			// Warten, bis die Parameter eingetroffen sind.
			//
			// TODO:
			// berprfen, ob die Verbindung gekappt wurde. Falls dem
			// so sein sollte, starten wir den Server einfach neu.
			
			while( mod_tcp.has_msg() < hdr.get_size() );
			
			buf = NULL;
			buf = new char[ hdr.get_size() + 1 ];
			memset( buf, 0, hdr.get_size() + 1 );
			mod_tcp.recv_msg( buf, hdr.get_size() );

			saved_net_msg msg;

			unsigned int tries = 0;
			bool error = false;
         		
			//Optimierung Anfang
         		int lost_cmd = 0;
         		int lost_param = 0;
         		bool cmds_cut = false;
         		//Optimierung Ende

			switch( hdr.get_type() )	
			{ // Begin switch(...)
				case MSG_TYPE_SYSTEM:
					
					// "Schneiderlein"-Befehle abfangen
					if( hdr.get_cmd() == MSG_CODE )
					{
						mod_robocom.mitte_ordinem( buf );
						mod_robocom.obsequere_ordini();
						conversation_status = CONVERSATION_READ_EXECUTED_COMMANDS;
						reanimate = true;
						break;
					}

					switch( hdr.get_cmd() )
					{ // Begin switch( hdr.get_cmd() )

						// Pings werden einfach verworfen
						case MSG_PING:
							continue;
							break;
						
						// Das Roboterprogramm soll neu aufgespielt werden
						case MSG_ANIMA:
							if( !mod_robocom.anima() )
								module_master::ptr().broadcast( "*Server: Roboterprogramm konnte nicht bertragen werden.\n" );
							else
								module_master::ptr().broadcast( "#Server: Roboter wurde erfolgreich animiert.\n" );	
							continue;
							break;
						
						// Der Roboter soll reanimiert werden
						case MSG_REANIMA:
							if( !mod_robocom.reanima() )
								module_master::ptr().broadcast( "*Server: Roboter konnte nicht reanimiert werden.\n" );
							else
								module_master::ptr().broadcast( "#Server: Roboter wurde erfolgreich reanimiert.\n" );

							continue;
							break;
						
						// Der Roboter soll abgeschaltet werden
						case MSG_INANIMA:
							if( !mod_robocom.inanima() )
								module_master::ptr().broadcast( "*Server: Der Roboter konnte nicht in den Schlafmodus versetzt werden.\n" );
							else
								module_master::ptr().broadcast( "#Server: Der Roboter wurde erfolgreich in den Schlafmodus versetzt.\n" );	

							continue;
							break;
						
						// Die "Schlafzeit" soll gesetzt werden
						case MSG_EXECUTION_TIME:
							sleep_time = static_cast<unsigned int>( round( atof( buf ) ) );
							module_master::ptr().broadcast( static_cast<string>( "#Server: Die geschtzte Zeit zur Ausfhrung der Befehle betrgt " + static_cast<string>( buf ) + " Sekunden.\n" ).c_str() );
							break;
						
						// Unbekannte Nachrichten werden ebenso ignoriert.
						default:
							continue;
							break;
								
					} // Ende switch( hdr.get_cmd() )

					break;
				
				case MSG_TYPE_ROBOT:
				
#ifdef DEBUG
					fprintf( stderr, "type = %i\n", hdr.get_type() );
					fprintf( stderr, "content = %i\n", hdr.get_cmd() );
					fprintf( stderr, "cksum = %i, calc = %i\n", hdr.get_chksum(), hdr.calc_checksum_mod() );
#endif

					// Startnachrichten werden gesondert behandelt. 
					if( 	hdr.get_cmd() == MSG_START_PHASE_1 ||
						hdr.get_cmd() == MSG_START_PHASE_2  )
					{
						switch( conversation_status )
						{
							case CONVERSATION_READ_ARCHIVED_COMMANDS:
								
								// Dieser Status sollte eigentlich immer ignoriert werden, es
								// sei denn, der Benutzer will, dass das Programm in hchstem
								// Mae geschwtzig ist :-)
								
								conversation_status = CONVERSATION_READ_RECEIVED_COMMANDS;
								
#ifdef DEBUG
								datalog = mod_robocom.accipe_nuntios( true );
								fprintf( stderr, "*** DATALOG DER ALTEN KOMMANDOS ***\n"
										 "%s\n"
										 "***********************************\n",
										 datalog.c_str() );
#endif

								break;

							case CONVERSATION_READ_RECEIVED_COMMANDS:

								datalog = "";
								datalog = mod_robocom.accipe_nuntios( true );

								// Befehle so lange senden, bis sie tatschlich empfangen wurden.
							
								tries = 0;
								error = false;	
                        					cmds_cut=false;
                        
								while( ( error = !check_command_datalog( received_messages, datalog, &mod_conf, &events, &lost_cmd, &lost_param, &cmds_cut ) ) && tries < SENDEVERSUCHE - 1)
								{ // Begin while(...)

									fprintf( stderr, "." );
									unsigned int cmd = 0;

									// Alle Aktionen abbrechen und den Roboter
									// wiederbeleben.
                           
									if( !cmds_cut || (lost_cmd==0 && lost_param<1) )
									{ 
                              							mod_robocom.mitte_iussum( MSG_HALT );
										sleep( SLEEP_TIME );
                              							mod_robocom.reanima();
                              							sleep( SLEEP_TIME );
                              							
                              							cmds_cut = false;
                           						}
                           
									datalog = "";
				   
									size_t i = 0;
									if ( cmds_cut )
										i=lost_cmd;
				   
									for( ; i < received_messages.size(); i++ )
									{ // Begin for(...)
										cmd = received_messages[ i ].header.get_cmd();
										if ( (!cmds_cut) || (lost_param==-1) || ((lost_param!=-1) && (lost_cmd!=i)) )
										{
											mod_robocom.mitte_iussum( static_cast<unsigned char>( cmd ) );
											sleep( SLEEP_TIME );
										}
				      
										size_t j=0;         
										if (lost_param==-1)
											lost_param=0;
										if ( cmds_cut && lost_cmd==i )
											j=lost_param;

										for( ; j < received_messages[ i ].header.get_size(); j++ )
										{
											mod_robocom.mitte_iussum( static_cast<unsigned char>( received_messages[ i ].params[ j ] ) );
											sleep( SLEEP_TIME );
										}

									} // Ende for(...)

									datalog = mod_robocom.accipe_nuntios( true );
									tries++;
                           
									cmds_cut=false;
									lost_cmd=0;

								} // Ende while(...)

								if( error )
								{
									module_master::ptr().broadcast( "*Server: Nach mehrmaligem Senden hat der Roboter immer noch nicht alle Befehle verstanden. Schade.\n" );
									error = false;
								}	

									
								conversation_status = CONVERSATION_READ_EXECUTED_COMMANDS;
								
								// TODO:
								// Eventuell hier received_messages lschen.
								break;

							default:
								break;
						}						

					}
					
					// Befehle senden. Aber nur, wenn es keine Fehler gab.
					
					if( error )
					{
						error = false;
						
						// Gespeicherte Parameter lschen.
						if( buf )
							delete[] buf;

						continue;
					}
					 
					sleep( SLEEP_TIME );
					mod_robocom.mitte_iussum( hdr.get_cmd() );
					
					msg.header = hdr;
					msg.params = "";
		
					// Wenn es Parameter gibt, so mssen diese natrlich
					// auch ber den IR-Tower versandt werden.
					if( hdr.get_size() > 0 )	
					{
						msg.params = static_cast<string>( buf );
						for( size_t i = 0; i < hdr.get_size(); i++ )
						{	
							sleep( SLEEP_TIME );
							mod_robocom.mitte_iussum( static_cast<unsigned char>( buf[ i ] ) );
						}
					}

					received_messages.push_back( msg );
					fprintf( stderr, "received_message.push_back:\n\tmsg = %i \n", msg.header.get_cmd() );
					
					for( int i = 0; i < msg.header.get_size(); i++ )
					{
						fprintf( stderr, "\tparam[ %i ] = %i\n", i, msg.params [ i ] );
					}

					// Gespeicherte Parameter lschen.
					if( buf )
						delete[] buf;

					/*

					Dieser Teil des Codes kommt unten nochmals vor. Es ist noch nicht
					hinreichend getestet worden, ob dies Probleme bereitet.

					// Wenn wir im Status CONVERSATION_READ_EXECUTED_COMMANDS sind,
					// sollten wir das Datalog abfragen, um zu berprfen, welche von
					// unseren Kommandos mit genau welchen Parametern ausgefhrt wurden.
					if( conversation_status == CONVERSATION_READ_EXECUTED_COMMANDS )
					{
						conversation_status = CONVERSATION_READ_ARCHIVED_COMMANDS;
						received_messages.clear();
					
						sleep( sleep_time + 1 );
						datalog = "";
						datalog = mod_robocom.accipe_nuntios( true );
					
						tries = 0;
						error = false;	
						while( datalog.find( mod_conf.da_datum( "AUSFUEHRUNGSPROTOKOLL_ENDE" ) ) == string::npos && tries < SENDEVERSUCHE - 1 )
						{
							sleep( SLEEP_TIME );
							
							datalog = "";
							datalog = mod_robocom.accipe_nuntios( true );
							tries++;
						};
						
						net_msg_hdr executed_commands( MSG_TYPE_SYSTEM, MSG_EXECUTED_COMMANDS, datalog.size() );
						mod_tcp.send_msg( reinterpret_cast<const char*>( &executed_commands ), sizeof( net_msg_hdr ), 0 );
						mod_tcp.send_msg( datalog.c_str(), datalog.size() );

						fprintf( stderr, 	"*** DATALOG MIT AUSGEFHRTEN KOMMANDOS: ***\n"
									"%s\n"
									"*******************************************\n",
									datalog.c_str() );
					}
					*/
					
					break;
					
				// Unbekannter Nachrichtentyp. Das ignorieren wir einfach mal...		
				default:
					 break;	

			} // Ende switch(...)

			// Wenn wir im Status CONVERSATION_READ_EXECUTED_COMMANDS sind,
			// sollten wir das Datalog abfragen, um zu berprfen, welche von
			// unseren Kommandos mit genau welchen Parametern ausgefhrt wurden.
			if( conversation_status == CONVERSATION_READ_EXECUTED_COMMANDS )
			{
				conversation_status = CONVERSATION_READ_ARCHIVED_COMMANDS;
					
				sleep( sleep_time + 1 );
				datalog = "";
				datalog = mod_robocom.accipe_nuntios( true );
			
				tries = 0;
				error = false;	
				while( datalog.find( mod_conf.da_datum( "AUSFUEHRUNGSPROTOKOLL_ENDE" ) ) == string::npos && tries < SENDEVERSUCHE - 1 )
				{
					sleep( SLEEP_TIME );
					
					datalog = "";
					datalog = mod_robocom.accipe_nuntios( true );
					tries++;
				}

				// ACHTUNG: Debug beginnt hier

				// Das Datalog wurde noch nicht richtig empfangen. Daher wird ein bewusst fehlerhaftes
				// Datalog gesendet, damit das GUI etwas zum Bearbeiten hat.
				if( datalog.find( mod_conf.da_datum( "AUSFUEHRUNGSPROTOKOLL_ENDE" ) ) == string::npos )
				{
					datalog = (string)( "888\n" ) +
						  (string)( "7800\n" ) +
						  mod_conf.da_datum( "AUSFUEHRUNGSPROTOKOLL_ANFANG" )+"\n";

					// Dieses Datalog enthlt alle Kommandos, die empfangen wurden
					for( size_t i = 0; i < received_messages.size(); i++ )
					{
						conv.clear();
						conv.str( "" );
						conv << static_cast<unsigned int>( received_messages[ i ].header.get_cmd() );
						if( 	received_messages[ i ].header.get_cmd() != MSG_START_PHASE_1 &&
							received_messages[ i ].header.get_cmd() != MSG_START_PHASE_2 &&
							received_messages[ i ].header.get_cmd() != MSG_PREPARE )
							datalog += conv.str() + "\n";
					}
				}

				// ACHTUNG: Debug endet hier
				
				fprintf( stderr, "received_message.clear()\n" );
				received_messages.clear();
				
				net_msg_hdr executed_commands( MSG_TYPE_SYSTEM, MSG_EXECUTED_COMMANDS, datalog.size() );
				mod_tcp.send_msg( reinterpret_cast<const char*>( &executed_commands ), sizeof( net_msg_hdr ), 0 );
				mod_tcp.send_msg( datalog.c_str(), datalog.size() );

				fprintf( stderr, 	"*** DATALOG MIT AUSGEFHRTEN KOMMANDOS: ***\n"
							"%s\n"
							"*******************************************\n",
							datalog.c_str() );

				// Dieses Flag sollte immer gesetzt werden, wenn am Ende eines Ausfhrungsblockes
				// der Roboter reanimiert werden muss.
				if( reanimate )
				{
					mod_robocom.reanima();
					reanimate = false;
				}
			}
		}

		// In regelmigen Abstnden fragt der Server bei seinem Client
		// an, ob die Verbindung noch besteht. Ist dies nicht der Fall,
		// so geht der Server einfach wieder in den Warten-Zustand ber
		// und erwartet eine neue Verbindung.
		if( difftime_msec( time_now, time_tmp ) > 2000 )
		{
			gettimeofday( &time_now, NULL );
			
			mod_tcp.send_msg( reinterpret_cast<char*>( &ping ), sizeof( net_msg_hdr ), MSG_NOSIGNAL );
		}

		usleep( 100000 );
	}
	
	return( 0 );
}
