/*
	~~~~~~~~~~~~
	~ module.h ~
	~~~~~~~~~~~~

Funktionen zur Modulverwaltung. Ebenso finden sich hier die Grundfunktionen der
Modulklasse.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/

#ifndef MODULE_H
#define MODULE_H

#include <vector>

using namespace std;

/// Die maximale Lnge des Modulnamens.
const int MODULE_NAME_LENGTH = 30;

/// module ist die abstrakte Basisklasse, die berall im Programm
/// verwendet wird. Von ihr sind alle tatschlich implementierten
/// Module abgeleitet.
class module
{
	public:

		/*

		Momentan keine Verwendung.

		/// Liefert die ID eines Moduls. Wie diese eineindeutige
		/// Identifikation erzeugt wird, ist von den Implementierungen
		/// der Module abhngig. Denkbar sind z.B. Zeitstempel, Zhlen
		/// der Instanzen etc.
		int get_id( void )
		{
			return( id );
		}

		*/
	
		virtual int get_status( void ) = 0;
		virtual char* get_name( void ) = 0;

	protected:

		virtual int mod_load( void ) = 0;
		virtual int mod_unload( void ) = 0;
		virtual int handle_msg( const char* msg ) = 0;

		/*
		Momentan keine Verwendung. Siehe oben.

		int id;
		*/

		/// Nur der Mediator darf Parameter ndern.
		friend class module_master;
};


/// module_info ist eine Hilfstruktur, die Informationen ber ein
/// bestimmtes Modul enthlt. Ein Vektor dieser Strukturen wird als
/// Datenbank des Mediators verwendet.
struct module_info
{
	char name[ MODULE_NAME_LENGTH + 1 ]; /// Der Name darf hchstens MODULE_NAME_LENGTH Zeichen lang sein.
	module* ptr;
	
	/// Wird momentan nicht verwendet. Eventuell zur eindeutigen Unterscheidung verschiedener Instanzen eines Moduls.
	int id;	
};
	
/// module_master ist die Klasse, die alle Daten ber alle momentan
/// geladenen Module enthlt. Diese Klasse ist ein Singleton, d.h.
/// es gibt nur eine Instanz fr jeden Programmlauf. Genaueres ist
/// in der Projektdokumentation zu finden.
class module_master
{
	public:
			
		int register_module( module *m );	
		int unregister_module( module *m );	
		int unload_all( void );			

		int broadcast( const char* msg );

		module* get_module( const char* name, int id = -1 );
		module* get_module( int id );

		static module_master& ptr( void );

	private:
		module_master( void );
		module_master( const module_master& );

		vector<module_info> db;
};

#endif
