///NQC-Kompilierer fr Linux
/*

      ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      ~ m o d u l e _ c o m p _ l i n u x n q c . c p p ~
      ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Klasse von Modulen zur Kompilierung von NQC-Roboterprogrammen, unter Verwendung
des Linux-NQC-Kompilierers. (http://bricxcc.sourceforge.net/nqc/)

Nahezu alle Konstrukte tragen lateinische Bezeichnungen. Latein ist neben 
Englisch womglich die einzige Weltsprache, die sich korrekt mit den ersten 128 
ASCII-Zeichen schreiben lt.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/


#ifndef MODULE_COMP_LINUXNQCCPP
#define MODULE_COMP_LINUXNQCCPP


#define PATRIA "./"


#include "module_comp_linuxnqc.h"



///Standardkonstruktion:
module_comp_linuxnqc::module_comp_linuxnqc()
{
   via_nqc = VIA_NQC;
   via_fun_r = PATRIA;
   via_fun_c = PATRIA;
   status = 0;
}

///Konstruktion unter Bercksichtigung einiger Pfade:
module_comp_linuxnqc::module_comp_linuxnqc(string via_confunditoris,
                                           string via_fundaminis_rudis,
                                           string via_fundaminis_confusi)                                          
{
   via_nqc = via_confunditoris;
   via_fun_r = via_fundaminis_rudis;
   via_fun_c = via_fundaminis_confusi;
   status = 0;
}

module_comp_linuxnqc::module_comp_linuxnqc(char* via_confunditoris,
                                           char* via_fundaminis_rudis,
                                           char* via_fundaminis_confusi)
{
   via_nqc = (string)via_confunditoris;
   via_fun_r = (string)via_fundaminis_rudis;
   via_fun_c = (string)via_fundaminis_confusi;
   status = 0;
}


int module_comp_linuxnqc::mod_load()
{
   lector = module_master::ptr().get_module("module_conf", -1);
   machina = (string)(((module_conf*)lector)->da_datum("ROBOTERTYP"));
   turris = (string)(((module_conf*)lector)->da_datum("SENDER")); 
   status = 1;
   return 0;
}


int module_comp_linuxnqc::mod_unload()
{
   remove((via_fun_r + (string)FUNDAMEN + (string)".temp.nqc").c_str());
   status = 0;
   return 0;
}


int module_comp_linuxnqc::handle_msg(const char* nuntius)
{
   //nihil faciendum est.
   return 0;
}


int module_comp_linuxnqc::get_status()
{
   return status;
}


char* module_comp_linuxnqc::get_name()
{
   return "module_comp_linuxnqc";
}


///Bleibt vorerst unimplementiert.
bool module_comp_linuxnqc::transscribe_data(char* vertendum)
{
   return transscribe_data((string)vertendum);
}

///Bleibt vorerst unimplementiert.
bool module_comp_linuxnqc::transscribe_data(string vertendum)
{
   module_master::ptr().broadcast(((string)"#" + (string)get_name()
                                 + (string)": Die Funktion ,,transscribe_data'' ist nicht implementiert.").c_str());
   return false;
}


///Grundlegende Kompiliermethode (ohne Transformationen des Quelltextes):
///@ param praeceptum Pfad und Dateiname des zu kompilierenden Quelltextes
///@ return Erfolg/Mierfolg
bool module_comp_linuxnqc::confunde(char* praeceptum)
{
   return confunde((string)praeceptum);
}

bool module_comp_linuxnqc::confunde(string praeceptum)
{
   //Zuknftig zu erledigen: Behandlung der NQC-Meldungen/-Rckgabewerte

   int rescriptum = system( (via_nqc + (string)"nqc "
                            + machina + (string)" "
                            + praeceptum).c_str() );
                            
   return proba_rescriptum_nqc(rescriptum);
}


///Stellt das Basisprogramm des Roboters zusammen und kompiliert es.
///@ param b Standardspannung, unverifiziert
///@ return Erfolg/Mierfolg
bool module_comp_linuxnqc::confunde_fundamen()
{
   float b = 8.000;
   module_master::ptr().broadcast(((string)"!" + (string)get_name() 
                                 + (string)"Das Roboterhauptprogramm wird fr eine Batteriespannung von 8 Volt optimiert.").c_str());
   return confunde_fundamen(b);  
}


///Stellt das Basisprogramm des Roboters zusammen und kompiliert es, wobei die
///Batteriespannung, fr welche das Programm optimiert wurde, in der Variable
///B gespeichert wird.
///@ return Erfolg/Mierfolg
bool module_comp_linuxnqc::confunde_fundamen(float& B)
{
   string data_aux = ".organon.nqc";

   flumen.open((via_fun_r + data_aux).c_str(), ios_base::out|ios_base::trunc);
   if(! flumen)
   {
      module_master::ptr().broadcast(((string)"*" + (string)get_name()
                                    + (string)": Die Textdatei "
                                    + via_fun_r + data_aux
                                    + (string)"konnte nicht angelegt werden.").c_str());
      return false;
   }
   
   flumen << ( (string) " task main()                         "
             + (string) " {                                   "
             + (string) "    while(true)                      "
             + (string) "    {                                "
             + (string) "       CreateDatalog(0);             "
             + (string) "       CreateDatalog(1);             "
             + (string) "       AddToDatalog(BatteryLevel()); "
             + (string) "       Wait(10);                     "
             + (string) "    }                                "
             + (string) " }                                   " );

   flumen.close();
   flumen.flush();
   flumen.clear();

   int rescriptum = system((via_nqc + (string)"nqc " + machina + (string)" " + turris 
                          + (string)" -d " + via_fun_r + (string)data_aux).c_str());
   if(!proba_rescriptum_nqc(rescriptum))
   {
      module_master::ptr().broadcast(((string)"#" + (string)get_name()
                                    + (string)": Die Batteriespannung des Roboters konnte nicht ermittelt werden."
                                    + (string)" Er wird womglich nicht auf die gewnschte Weise auf Befehle reagieren.").c_str());
      return false;
   }

   sleep(1);

   rescriptum = system((via_nqc + (string)"nqc " + turris + (string)" -run").c_str());
   if(!proba_rescriptum_nqc(rescriptum))
   {
      module_master::ptr().broadcast(((string)"#" + (string)get_name()
                                    + (string)": Die Batteriespannung des Roboters konnte nicht ermittelt werden."
                                    + (string)" Er wird womglich nicht auf die gewnschte Weise auf Befehle reagieren.").c_str());
      return false;
   }

   sleep(1); //necessarium est?

   rescriptum = system((via_nqc + (string)"nqc " + turris 
                      + (string)" -datalog > "
                      + via_fun_r + data_aux).c_str());
   if(!proba_rescriptum_nqc(rescriptum))
   {
      module_master::ptr().broadcast(((string)"#" + (string)get_name()
                                    + (string)": Die Batteriespannung des Roboters konnte nicht ermittelt werden."
                                    + (string)" Er wird womgich nicht auf die gewnschte Weise auf Befehle reagieren.").c_str());
      return false;
   }

   flumen.open((via_fun_r + data_aux).c_str(), ios_base::in);
   if(! flumen)
   {
      module_master::ptr().broadcast(((string)"*" + (string)get_name()
                                    + (string)": Die Textdatei "
                                    + via_fun_r + data_aux
                                    + (string)"konnte nicht gefnet werden.").c_str());
      return false;
   }

   string str_temp;
   while(! flumen.eof())
   {
      str_temp += flumen.get();
   }
   istringstream influmen_lit(str_temp);
   influmen_lit >> B;

   module_master::ptr().broadcast(((string)"!Batteriespannung: " + str_temp + (string)"mV").c_str());

   flumen.close();
   flumen.flush();
   flumen.clear();   

   flumen.open((via_fun_r + (string)FUNDAMEN + (string)".nqc").c_str());
   if(!flumen)
   {
      module_master::ptr().broadcast(((string)"*" + (string)get_name()
                                    + (string)": Das Basisprogrammgerst des Roboters ("
                                    + via_fun_r + (string)FUNDAMEN
                                    + (string)") wurde nicht gefunden.").c_str());
      return false;
   }

   string fundamen = "/* ~ ~ ~ T e m p o r  r e   D a t e i ~ ~ ~\n\n nderungen sind zwecklos, werden vom Programm berschrieben.*/\n\n\n\n\n\n\n\n\n\n\n\n\n";
   while(!flumen.eof())
   {
      fundamen += flumen.get();
   }


   //interpretatio et perfectio:
   	
   string::size_type locus=0;		
   string tmp_str = "";
   string f_intro=fundamen;
   f_intro.erase(0,f_intro.find("//DEFINE-ANFANG")+strlen("//DEFINE-ANFANG")+1);
   f_intro.erase(f_intro.find("//DEFINE-ENDE"+1) );
   string befehl="";
   
   ostringstream conv;
   
   //Abhngige Defines einsetzen:
   vector<double> temp;
   temp.clear();
   temp.push_back((double)B);
   
   conv.flush();
   conv.clear();
   conv.str("");
   //Zuknftig zu erledigen: Gehfaktor auf die Auflsung der Karte zuschneidern
   double GENAUIGKEIT = atoi( ((module_conf*)lector)->da_datum( "GENAUIGKEIT" ) );
   string METERZEIT = ( ((module_conf*)lector)->da_datum( "METERZEIT" ) );
   string PERIODENDAUER = ( ((module_conf*)lector)->da_datum( "PERIODENDAUER" ) );

   double meterzeit_erg = ( term_terminate( METERZEIT ) )[ 0 ].precisely_evaluate( temp );
   double periodendauer_erg = ( term_terminate( PERIODENDAUER ) )[ 0 ].precisely_evaluate( temp );
   double EPM = atof( ( ((module_conf*)lector)->da_datum( "EPM" ) ) );

   conv << static_cast<unsigned int>( round( meterzeit_erg * GENAUIGKEIT / EPM ) );

   string gehfaktor = conv.str();
   conv.flush();
   conv.clear();
   conv.str("");

   conv << static_cast<unsigned int>( round( periodendauer_erg * GENAUIGKEIT / 360.0  ) );

   string drehfaktor = conv.str();
   conv.flush();
   conv.clear();
   conv.str("");
   
   conv << static_cast<unsigned int>( round( periodendauer_erg * GENAUIGKEIT / 4.0  ) );

   string drehfaktor90 = conv.str();  
   conv.flush();
   conv.clear();
   conv.str("");
 
   fundamen.replace(fundamen.find("@@~~^^$$ABHAENGIGE_DEFINES_EINFUEGEN$$^^~~@@"),((string)("@@~~^^$$ABHAENGIGE_DEFINES_EINFUEGEN$$^^~~@@")).length(),(string)""
        +(string)"#define GEHFAKTOR "+gehfaktor+(string)"\n"
        +(string)"#define DREHFAKTOR "+drehfaktor+(string)"\n"
        +(string)"#define DREHFAKTOR90 "+drehfaktor90+(string)"\n");
   
   f_intro.erase(0,f_intro.find("#define "));
   
   while ( f_intro.find("#define") != string::npos )
   {
       f_intro.erase(0,f_intro.find("#define ") + strlen("#define ") );
       f_intro.erase(0,f_intro.find_first_not_of((string) "\n\t "));
       befehl=f_intro.substr(0,f_intro.find_first_of((string)"\n\t "));
      
      //Defines vervollstndigen
      locus = fundamen.find("#define "+befehl) + ((string)"#define "+befehl).length();
      fundamen.insert(locus, ((string)" " + (string)((module_conf*)lector)->da_datum(befehl.c_str())));
            
      if( ((module_conf*)lector)->estne_datum( ("DIM_"+befehl).c_str() ) )   
      {   
         //Switch vervollstndigen
         int dim=atoi(((module_conf*)lector)->da_datum(("DIM_"+befehl).c_str()));
         tmp_str = "";
         tmp_str.append((string)"\n\tcase " + befehl +":\n" );
         tmp_str.append((string)"\t\tAddToDatalog(" + befehl+");\n");
         tmp_str.append((string)"\t\tClearTimer(BEFEHLUHR);\n");
         tmp_str.append((string)"\t\tDO_" + befehl + "(");
         for (int j=0; j<dim; j++)
         {
            tmp_str.append((string)"temp[++j]");
            if (j<dim-1)
            { 
               tmp_str.append((string)", "); 
            }
         }
         tmp_str.append((string)");\n\t\t++j;\n\t\tbreak;\n\n");
         locus = fundamen.find("@@~~^^$$CASES_EINFUEGEN$$^^~~@@");
         fundamen.insert(locus, tmp_str);
         
          
         tmp_str = "";
         tmp_str.append((string)"\nvoid DO_" + befehl+ "(" );
         for (int j=0; j<dim; j++)
         {
            conv.flush();
            conv.clear();
            conv.str("");
            conv << j;
            tmp_str.append((string)"int p_"+conv.str());
            if (j<dim-1)
            { tmp_str.append((string)", "); }
         }
         tmp_str.append((string)")\n{\n" + (string)(((module_conf*)lector)->da_datum(((string)"1_NQC_"+befehl).c_str()))+(string)"\n}\n");
         locus = fundamen.find("@@~~^^$$FUNKTIONEN_EINFUEGEN$$^^~~@@");
         fundamen.insert(locus, tmp_str);
      }
   }
   
   fundamen.erase(fundamen.find("@@~~^^$$CASES_EINFUEGEN$$^^~~@@"), strlen("@@~~^^$$CASES_EINFUEGEN$$^^~~@@"));
   fundamen.erase(fundamen.find("@@~~^^$$FUNKTIONEN_EINFUEGEN$$^^~~@@"), strlen("@@~~^^$$FUNKTIONEN_EINFUEGEN$$^^~~@@"));
  

   //confusio:

   flumen.close();
   flumen.clear();
   
   flumen.open((via_fun_r + (string)FUNDAMEN
                + (string)".temp.nqc").c_str(), ios_base::out|ios_base::trunc);
   if(! flumen)
   {
      module_master::ptr().broadcast(((string)"*" + (string)get_name()
                                    + (string)": Eine Textdatei"
                                    + via_fun_r + (string)FUNDAMEN + (string)".nqc.temp"
                                    + (string)" wird ben�igt.").c_str());
      return false;
   }

   flumen << fundamen;
   flumen.close();
   flumen.flush();

   bool finis = confunde((via_fun_r + (string)FUNDAMEN + (string)".temp.nqc").c_str());
   
   return finis;
}



#undef PATRIA

#endif


