///NQC-Kompilierer fr Linux
/*

        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ~ m o d u l e _ c o m p _ l i n u x n q c . h ~
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Klasse von Modulen zur Kompilierung von NQC-Roboterprogrammen, unter Verwendung
des Linux-NQC-Kompilierers. (http://bricxcc.sourceforge.net/nqc/)

Alle Konstrukte tragen lateinische Bezeichnungen. Latein ist neben Englisch
womglich die einzige Weltsprache, die sich korrekt mit den ersten 128 ASCII-
Zeichen schreiben lt.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/


#ifndef MODULE_COMP_LINUXNQCH
#define MODULE_COMP_LINUXNQCH


#include "module_comp.h"
#include "module_linuxnqc.h"
#include "module_conf.h"
#include "sim_language.h"

#include <string>
#include <fstream>
#include <sstream>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>


using namespace std;



///Konstrukt zur Kompilierung von NQC-Quelltexten (unter Linux). Insbesondere
///bietet die Klasse eine Funktion zum Kompilieren des Basisprogramms des 
///Roboters.
///@param via_nqc Pfad zum NQC-Programm
///@param via_fun_r Pfad zum Roboterhauptprogrammgerüst
///@param via_fun_c Pfad zum kompilierten Roboterhauptprogramm
///@param machina Robotertyp-Option fr nqc (z. B. ,,-Trcx2")
///@param turris Sendegert-Option fr nqc (z. B. ,,-Susb")
//classis definita
struct module_comp_linuxnqc :
  public module_comp, public module_linuxnqc
{
  protected:
   //via confunditoris:
   string via_nqc; ///<Pfad zum nqc-Kompilierer
   //via fundaminis rudis:
   string via_fun_r; ///<Pfad zum Hauptprogrammgerst
   //via fundaminis confusi:
   string via_fun_c; ///<Pfad zum kompilierten Hauptprogramm

   string machina; //e. g. "-Trcx2"
   string turris; //e. g. "-Susb"
   
   fstream flumen;

   int status;

   virtual int mod_load();
   virtual int mod_unload();
   virtual int handle_msg(const char*);

   ///bersetzt abstrakten Quelltext aus der Roboterdatei in NQC (vorerst 
   ///unimplementiert)
   //transformat litteras fontanas datorum lectoris (module*) in NQC. 
   //ista functio impleta non est.
   virtual bool transscribe_data(char*);
   bool transscribe_data(string);

  public:
   ///Konstruktoren:
   module_comp_linuxnqc();
   module_comp_linuxnqc(string,string,string);
   module_comp_linuxnqc(char*,char*,char*);

   ///(lat.: gib_Pfad_zum_kompilierten_Basisprogramm)
   //dat via fundaminis confusi:
   virtual char* da_via_fun_c()
    { return (char*)(via_fun_c.c_str()); }

   virtual int get_status();
   virtual char* get_name();

   ///Grundlegende Kompiliermethode (ohne Transformation des Quelltextes)
   //confundet ordinem:
   virtual bool confunde(string);
   virtual bool confunde(char*);

   ///Kompilation des Roboterhauptprogramms
   //confundet ordinem principalis (,,fundamen") in loco ,,da_via_fun_c" rescripto:
   virtual bool confunde_fundamen(float&);
   virtual bool confunde_fundamen();
};



#endif
