///Roboterdateileser auf ConfigFile-Basis
/*

     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     ~ m o d u l e _ c o n f _ C o n f i g F i l e . c p p ~
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Eine Modulklasse zur Verwaltung von Programmoptionsdateien. Basiert auf 
ConfigFile (Lizenz: MIT) von Richard J. Wagner, dem fr sein Werk hiermit 
herzlich gedankt sei. Siehe: 

   http://www-personal.umich.edu/~wagnerr/ConfigFile.html

Alle Konstrukte tragen lateinische Bezeichnungen. Latein ist neben Englisch
womglich die einzige Weltsprache, die sich korrekt mit den ersten 128 ASCII-
Zeichen schreiben lt.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/


#ifndef MODULE_CONF_CONFIGFILECPP
#define MODULE_CONF_CONFIGFILECPP


#include "module_conf_ConfigFile.h"


#define VIA_DATORUM "./"
#define NOMEN_DATORUM "Gingganz.conf"


using namespace std;



//functiones datorum interpretandum (qua transformata in horreo sunt):

string module_conf_ConfigFile::nulla(string s)
{
   return s;
}

string module_conf_ConfigFile::prima(string s)
{
   if(s[0] != '~')
      module_master::ptr().broadcast("#module_conf_ConfigFile: Scheinbar ist Syntaxfehler in der Konfigurationsdatei. (Eine gewisse Zeile sollte mit ,,~\" beginnen, tut es aber nicht.");
   else
      s.replace(0, 1, "");

   string::size_type locus = s.find("\n~");
   while (locus != string::npos)
   {
      s.replace(locus+1, 1, "");
      locus = s.find("\n~");
   }

   locus = s.find("@@~~^^$$aequumaequum$$^^~~@@");
   while (locus != string::npos)
   {
      s.replace(locus, 28, "=");
      locus = s.find("@@~~^^$$aequumaequum$$^^~~@@");
   }

   locus = s.find("@@~~^^$$cruxcrux$$^^~~@@");
   while (locus != string::npos)
   {
      s.replace(locus, 24, "#");
      locus = s.find("@@~~^^$$cruxcrux$$^^~~@@");
   }

   return s;
}

//secunda, tertia, quarta . . .


///Standardkonstruktion, Datei heit ,,Gingganz.conf" (nach einem Gedicht von
///Christian Morgenstern)
//constructio usitata, in qua ,,Gingganz.conf" data nominantur:
module_conf_ConfigFile::module_conf_ConfigFile()
{
   via_dat = (string)VIA_DATORUM + (string)NOMEN_DATORUM;
   interpretare[0] = &nulla;
   interpretare[1] = &prima;
   status = 0;
}


///Konstruktion mit alternativer Konfigurationsdatei
///@param via_datorum Pfad und Name der gewnschten Optionendatei
//constructio via datorum addita:
module_conf_ConfigFile::module_conf_ConfigFile(string via_datorum)
{
   via_dat = via_datorum;
   interpretare[0] = &nulla;
   interpretare[1] = &prima;
   status = 0;
}


///Konstruktion mit alternativer Konfigurationsdatei:
///@param via_datorum Pfad und Name der gewnschten Optionendatei
//constructio via datorum addita:
module_conf_ConfigFile::module_conf_ConfigFile(char* via_datorum)
{
   via_dat = (string)via_datorum;
   interpretare[0] = &nulla;
   interpretare[1] = &prima;
   status = 0;
}


int module_conf_ConfigFile::get_status()
{
   return status;
}


char* module_conf_ConfigFile::get_name()
{
   return "module_conf_ConfigFile";
}


int module_conf_ConfigFile::mod_load()
{
   try
   {
      data = new ConfigFile(via_dat);
   }

   catch(ConfigFile::file_not_found)
   {
      module_master::ptr().broadcast(((string)"*" + (string)get_name()
                                    + (string)": Die Konfigurationsdatei"
                                    + via_dat
                                    + (string)" wurde nicht gefunden.").c_str());
      status = -1;
      return -1;
   }

   status = 1;
   return 0;
}


int module_conf_ConfigFile::mod_unload()
{
   delete data;
   return 0;
}


int module_conf_ConfigFile::handle_msg(const char* nuntius)
{
   //nullum faciendum est.
   return 0;
}


///Liest eine Variable aus der Roboterdatei aus. Etwaige Transformationen
///geschehen vollautomatisch, wenn die Syntax der Datei eingehalten worden 
///ist.
///(lat.: gib_Gegebenes)
//leget et dat datum nominis additi ex datorum:
char* module_conf_ConfigFile::da_datum(const string nomen)
{
   string datum;

   //lectio pura
   if( ! data->readInto(datum, nomen))
   {
      module_master::ptr().broadcast(((string)"#" + (string)get_name()
                                    + (string)": " + (string)"Die Variable ,,"
                                    + nomen + (string)" steht nicht in der Konfigurationsdatei.").c_str());
      return "";
   }

   //interpretatio, si necessaria est:

   if(datum[0] == '@') //gnomon ad alterum datum
      return da_datum(datum.substr(1,string::npos));
   unsigned short i = 0;

   //datum ,,0_xxx" provocat functio interpretandi nulla, datum ,,1_xxx"
   //prima, et cetera.
   for(; i <= num_func; i++)
   {
      if(nomen[0] == (char)(48+i)) //48 = ASCII-"0"
      {
         datum = interpretare[i](datum);
      }
   }

   return (char*)datum.c_str();
}

char* module_conf_ConfigFile::da_datum(const char* nomen)
{
   return da_datum((string)nomen);
}


///Diese Funktion prft, ob ihr Argument eine Variable in der 
///Konfigurationsdatei ist.
///(lat.: ist_es_ein_Gegebenes)
//probat, si datum est nominis additi:
bool module_conf_ConfigFile::estne_datum(const char* incognitum)
{
   return data->keyExists((string)incognitum);
}

bool module_conf_ConfigFile::estne_datum(const string incognitum)
{
   return data->keyExists(incognitum);
}



#undef VIA_DATORUM
#undef NOMEN_DATORUM


#endif
