///Roboterdateileser auf ConfigFile-Basis
/*

        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ~ m o d u l e _ c o n f _ C o n f i g F i l e . h ~
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Eine Modulklasse zur Verwaltung von Programmoptionsdateien. Basiert auf 
ConfigFile (Lizenz: MIT) von Richard J. Wagner, dem fr sein Werk hiermit 
herzlich gedankt sei. Siehe: 

   http://www-personal.umich.edu/~wagnerr/ConfigFile.html

Alle Konstrukte tragen lateinische Bezeichnungen. Latein ist neben Englisch
womglich die einzige Weltsprache, die sich korrekt mit den ersten 128 ASCII-
Zeichen schreiben lt.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/


#ifndef MODULE_CONF_CONFIGFILE
#define MODULE_CONF_CONFIGFILE


#include <string>
#include "module_conf.h"
#include "ConfigFile.h"


using namespace std;



///Ausleseklasse fr Konfigurationsdateien, basierend auf ,,ConfigFile" von 
///Richard J. Wagner.
//classis definita
struct module_conf_ConfigFile :
  public module_conf
{
  private:
   //functiones datorum interpretandum (qua transformata in horreo sunt):
   ///Transformationsfunktionen fr Daten, die aus technischen Grnden nicht
   ///in ihrer eigentlichen Form in der Optionendatei stehen. (Die zum Beispiel
   ///Leerzeilen enthielten, was ConfigFile nicht untersttzt.) Zeiger auf diese
   ///Funktionen werden bei der Instanzenkonstruktion im Feld ,,interpretare"
   ///festgehalten (deswegen mssen sie statisch sein).
   static string nulla(string);  ///<(lat.: die erste [Funktion])
   static string prima(string);  ///<(lat.: die zweite [Funktion])
   //secunda, tertia, quarta . . .

   //numerus functionum:
   const static unsigned short num_func = 1; ///<Anzahl der Trafo-Funktionen,
                                             ///,,nulla" nicht eingerechnet

  protected:
   //via datorum:
   string via_dat; ///<Pfad und Name der Konfigurationsdatei
                   ///<(lat.: Pfad_zu_den_Gegebenen[_Dingen])
   //procurator datorum:
   ConfigFile* data; ///<Stellt alle Auslesefunktionen zur Verfgung.

   int status;

   //heredium:
   virtual int mod_load();
   virtual int mod_unload();
   virtual int handle_msg(const char*);

   //horreum functionum datorum interpretandum:
   typedef string (*functio)(string);
   ///Funktionen aus diesem Feld transformieren, wenn ntig, ausgelesene Datei-
   ///variablen, bevor da_datum diese zurckgibt.
   ///(lat.: bersetze)
   functio interpretare[num_func+1];

  public:
   //constructio usitata, in qua ,,Gingganz.conf" data nominantur:
   module_conf_ConfigFile(); ///<Standardkonstruktion, Datei heit 
                             ///,,Gingganz.conf" (nach einem Gedicht von
                             ///Christian Morgenstern):
   //constructio via datorum addita:
   module_conf_ConfigFile(string); ///<Pfad und Name der zu verwendenden Datei
                                   ///als Parameter.
   module_conf_ConfigFile(char*); ///<Pfad und Name der zu verwendenden Datei
                                  ///als Parameter.

   virtual int get_status();
   virtual char* get_name();

   ///Liest eine Variable aus der Roboterdatei aus. Etwaige Transformationen
   ///geschehen vollautomatisch, wenn die Syntax der Datei eingehalten worden 
   ///ist.
   ///(lat.: gib_Gegebenes)
   //leget et dat datum nominis additi ex datorum:
   virtual char* da_datum(const char*);
   char* da_datum(const string);
   
   ///Diese Funktion prft, ob ihr Argument eine Variable in der 
   ///Konfigurationsdatei ist.
   ///(lat.: ist_es_ein_Gegebenes)
   //probat, si datum est nominis additi:
   virtual bool estne_datum(const char*);
   bool estne_datum(const string);
};



#endif
