/*
	~~~~~~~~~~~~~~~~
	~ module_net.h ~
	~~~~~~~~~~~~~~~~

Abstraktes Basisklasse fr alle Netzwerkkommunikationsmodule.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/

#ifndef MODULE_NET_H
#define MODULE_NET_H

#include "module.h"

/// Basisklasse, die fr jedwede Netzwerkkommunikation verwendet
/// wird. Diese Klasse ist abstrakt; benutzbare Klassen mssen
/// daher erst abgeleitet werden.
class module_net : public module
{
	public:
	
	/// Dieser Aufruf von connect erstellt einen Server, der auf dem entsprechenden
	/// Port Verbindungen entgegennimmt.
	virtual int connect( unsigned int port ) = 0;	
	
	/// Dieser Aufruf von connect erstellet einen Client, der sich zu einer
	/// bestimmten Adresse auf einen bestimmten Port verbindet.
	virtual int connect( const char* server, unsigned int port ) = 0;
	virtual int disconnect( void ) = 0;

	virtual int send_msg( const char* msg, unsigned int size, int flags = 0 ) = 0;
	virtual int recv_msg( char* buf, unsigned int size, int flags = 0 ) = 0;

	virtual int has_msg( void ) = 0;
	virtual int get_status( void ) = 0;

	virtual bool has_connection( void ) = 0;

	virtual int mod_load( void ) = 0;
	virtual int mod_unload( void ) = 0;

	/// Gibt den Namen des Moduls zurck.
	///
	/// @return "abstract_module_net_do_not_use"
	char* get_name( void )
	{
		return( "abstract_module_net_do_not_use" );
	};

	protected:

	bool is_server;
	bool is_connected;
};

#endif
