/*
	~~~~~~~~~~~~~~~~~~~~
	~ module_net_tcp.h ~
	~~~~~~~~~~~~~~~~~~~~

Modul zur Netzwerkkommunikation ber TCP/IP. Das Modul kann sowohl einen 
Servermodus wie auch einen Clientmodus bieten.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/

#ifndef MODULE_NET_TCP_H
#define MODULE_NET_TCP_H

#include "module_net.h"

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>

/// Maximale Anzahl von Clients, die gleichzeitig eine Verbindung zum Server
/// offen haben drfen.
const unsigned int MODULE_NET_TCP_MAX_CLIENTS = 2;

/// Implementierung der Klasse module_net zur Verwendung mit TCP/IP.
class module_net_tcp : public module_net
{
	public:
		module_net_tcp( void );

		virtual int connect( unsigned int port );
		virtual int connect( const char* server, unsigned int port );
		virtual int disconnect( void );

		virtual int send_msg( const char* msg, unsigned int size, int flags = 0 );
		virtual int recv_msg( char* buffer, unsigned int size, int flags = 0 );
		
		virtual int has_msg( void );
		virtual int get_status( void );

		virtual bool has_connection( void );
		
		virtual int mod_load( void );
		virtual int mod_unload( void );
		
		virtual int handle_msg( const char* );
		
		/// Gibt den Namen des Moduls zurck.
		///
		/// @return "module_net_tcp"
		virtual char* get_name( void )
		{
			return( "module_net_tcp" );
		}

	private:
	
		int s; ///< Socket fr den Server 
		int c; ///< Socket fr den Client
		
		sockaddr_in sock_addr;
		sockaddr_in client_addr;
	
		/// Zhlt alle Instanzen des Moduls. Somit ist die Zuweisung einer eindeutigen
		/// ID mglich. Momentan nicht verwendet.	
		static unsigned int num_instances;	
};	

#endif
