#include <string>
#include <vector>
#include "module_parser.h"
#include <iostream>
#include "sim_language.h"

const string whites("\r\n\t ");

/// Wandelt einen Befehlsblock in Form von Elementarbefehlen, Makros
/// und "Mageschneiderten Befehlen" (unkompilierter NQC-Quelltext, noch nicht implementiert)
/// in einen Vektor aus saved_net_msg um, d.h. Paare von Netzwerk-Nachrichtenheadern (Typ, Kommando/Code, Parametergre)
/// und Strings (enthalten pro Zeichen einen Parameter).
/// Dazu werden zunchst alle Makros durch ihre durch sie reprsentierten Befehle ersetzt, wobei deren Parameter
/// von dem/den Parameter(n) des Makros abhngen knnen und daher hier berechnet werden.
/// Danach befinden sich im Text nur noch Elementarbefehle, die durch die Codes aus der Konfig-Datei identifiziert
/// werden.
///
/// @param input Der Befehlsblock, bestehend aus Elementarbefehlen, Makros und ihren Parametern (direkt aus der Nutzereingabe)
///
/// @return Der Vektor an saved_net_msg, mit allen Befehlen aus dem Text, oder ein leerer Vektor, falls
///         ein Befehl unbekannt war, oder die Anzahl der Parameter eines Befehls nicht mit der Definition
///         aus der Konfig-Datei bereingestimmt hat. In diesen Fllen wird auerdem ein Fehler ausgegeben.
vector<saved_net_msg> module_parser::parse(string input)
{
       unsigned int end;
       vector<saved_net_msg> output; 
       string input_cpy = input;
       //Weirume entfernen:
       while (input_cpy.find_first_of(whites)!=string::npos) {input_cpy.erase(input_cpy.find_first_of(whites), 1);}
       saved_net_msg tmp_pm;
       ostringstream conv;
       
       //Makros ersetzen:
       while (input_cpy.size()!=0)
       {
            /****Bezeichner-Modus****/
            //Bezeichner extrahieren: 
            end=input_cpy.find('(');
            string name=input_cpy.substr(0,end);
            string normal_name=name;
            input_cpy.erase(0,end+1);
            //Parameter extrahieren.
            end=input_cpy.find(')');
            string param_str=input_cpy.substr(0,end);
            input_cpy.erase(0,end+1);
         
            transform(name.begin(), name.end(), name.begin(), (int(*)(int)) toupper);				
				
            unsigned char cmd;
            if( mod_conf->estne_datum(((string)"MAKRO_" + name).c_str()) && (input.find(normal_name)!=string::npos) )
            {
               vector<unsigned char> params;
               params.clear();
               while (param_str.find_first_of(",;")!=string::npos)
               {
                  params.push_back((unsigned char)atoi(param_str.substr(0,param_str.find_first_of(",;")).c_str()));
                  param_str.erase(0,param_str.find_first_of(",;")+1);
               }
               params.push_back((unsigned char)atoi(param_str.c_str()));
               if( atoi( mod_conf->da_datum( ((string)"DIM_"+name.c_str()).c_str() ) ) != params.size() )
               {
                  module_master::ptr().broadcast( "*module_parser: Anzahl der Parameter ist nicht korrekt.\n" );
               }               
					string content_str=(string)mod_conf->da_datum(((string)"MAKRO_"+name).c_str());
					while (content_str.find_first_of(whites)!=string::npos) {content_str.erase(content_str.find_first_of(whites), 1);}
               vector<conf_command> makro_content = term_parse_cmd(content_str);
               end=input.find(normal_name);
               input.erase(end,normal_name.length());
					while (input[end]!=')') {input.erase(end,1);}
					input.erase(end,1);
               for (int i=makro_content.size()-1; i>=0; i--)
               {
                  conv << static_cast<int>( makro_content[i].formula.evaluate(params) );
                  input.insert(end,(makro_content[i].name+(string)"("+conv.str()+(string)")").c_str());
                  conv.clear();
                  conv.str("");
               }
            }
       }
		 
       //Eigentliches Parsen: 
       //Vorangehende Weirume entfernen:
       end=input.find_first_not_of(whites);
       input.erase(0,end);      
       while (input.size()!=0)
       {
       //Vorbereitungen:
       tmp_pm.header.set_type(MSG_TYPE_ROBOT);
       tmp_pm.header.set_cmd(14);
       tmp_pm.params=("");
       /****Bezeichner-Modus****/
       //Vorangehende Weirume entfernen:
       end=input.find_first_not_of(whites);
       input.erase(0,end);
       //Bezeichner extrahieren: 
       end=input.find('(');
       string name=input.substr(0,end);
       input.erase(0,end+1);
	   transform(name.begin(), name.end(), name.begin(), (int(*)(int)) toupper);

	   unsigned char cmd;
      if( mod_conf->estne_datum( name.c_str() ))
	   {
		   cmd = static_cast<unsigned char>( atoi( mod_conf->da_datum( name.c_str() ) ) );
		   tmp_pm.header.set_cmd( cmd );
	   }
	   else
	   {
		   module_master::ptr().broadcast( (((string)("*module_parser: Befehl nicht gefunden: \""+name+"\"\n")).c_str()) );
		   output.clear();
		   return( output );
	   }

	   mod_map->add_command( name, cmd );
           
      /****Parameter-Modus****/
	   if( cmd == MSG_CODE )
           {
                  //Eventuelle innere Klammern bergehen
                  end=0;
                  unsigned int ps=1; //Klammern-Zhler
                  while (ps!=0)
                  {
                        switch (input[end])
                        {
                               case '(': {++ps; break;}
                               case ')': {--ps; break;}
                        }
                        ++end;
                  }             
                  tmp_pm.params=input.substr(0,end-1);
                  tmp_pm.header.set_size(tmp_pm.params.size());
                  input.erase(0,end+1);
                  tmp_pm.header.set_type(MSG_TYPE_SYSTEM);
           }
           else
           {               
		   		  //Weirume innerhalb der Klammern entfernen:
                  while (input.find_first_of(whites)<input.find(")"))
                  {
                        input.erase(input.find_first_of(whites),1);
                  }   
                  //Parameter extrahieren:
                  end=0;
		  string tmp_string;
                  while ( input.find_first_of(",;") < input.find(')') )
                  {
                        end=input.find_first_of(",;");
                        unsigned char tmpchar=atoi(input.substr(0,end).c_str());
            			tmp_string = tmpchar;
            			tmp_pm.params.append( tmp_string );
                        input.erase(0,end+1);
                        if ((tmpchar==0) || (tmpchar>250))
        				  {
			                 module_master::ptr().broadcast( "*module_parser: Parameter zu gro oder Null.\n" );
			        		 output.clear();
	        				 return( output );
        		          }
                  }        
                  end=input.find(')');
                  unsigned char tmpchar=atoi(input.substr(0,end).c_str());
		  if( input.substr( 0, end ) != "" )
		  {
        		  tmp_string = tmpchar;
        		  tmp_pm.params.append( tmp_string );
        		  if ((tmpchar==0) || (tmpchar>250))
				  {
			         module_master::ptr().broadcast( "*module_parser: Parameter zu gro oder Null.\n" );
					 output.clear();
					 return( output );
 		          }
		  }
                  input.erase(0,end+1);
                  tmp_pm.header.set_size( tmp_pm.params.size() );

		          if( atoi( mod_conf->da_datum( ((string)"DIM_"+name.c_str()).c_str() ) ) != tmp_pm.header.get_size() )
				   {
					   module_master::ptr().broadcast( "*module_parser: Anzahl der Parameter ist nicht korrekt.\n" );
					   output.clear();
					   return( output );
				   }                  
           } 
           int tmp_int=output.size();
           for (int i=0; i<output.size(); i++)
           { tmp_int+=output[ i ].header.get_size(); }
           output.push_back(tmp_pm);  
           if (tmp_int>atoi(mod_conf->da_datum( "MAX_ANZAHL_BEFEHLE" )))
           {
			  	  module_master::ptr().broadcast( "*module_parser: Anzahl an Elementarbefehlen wurde überschritten.\n" );
			     output.clear();
				  return( output );
			  }
         //Anschlieende Weirume entfernen:
         end=input.find_first_not_of(whites);
         input.erase(0,end);
       } 
       return( output );    
}

/// Das Modul reagiert in dieser Version nicht auf Nachrichten.
int module_parser::handle_msg( const char* msg )
{
	return( 0 );
}

int module_parser::mod_load( void )
{
    mod_conf = dynamic_cast<module_conf*>( module_master::ptr().get_module( "module_conf" ) );
    MSG_CODE = atoi( mod_conf->da_datum( "MASZGESCHNEIDERTER_BEFEHL" ) );

	 mod_map = dynamic_cast<module_map*>( module_master::ptr().get_module( "module_map" ) );
    
    return( 0 );
}

int module_parser::mod_unload( void )
{
    return( 0 );
}

int module_parser::get_status( void )
{
   	return( 0 );
}
