///Roboterkommunikation via Linux-nqc
/*

       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
       ~ m o d u l e _ r o b o c o m _ l i n u x n q c . h ~
       ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Eine Modulklasse zur Roboterkommunikation, welche die Konsolenfunktionen des 
NQC-Kompilierers fr Linuxsysteme nutzt. (http://bricxcc.sourceforge.net/nqc/)
Funktioniert nur zusammen mit einem Kompiliermodul des Typs 
module_comp_linuxnqc, mit dem sie auch die Basisklasse module_linuxnqc teilt.

Alle Konstrukte tragen lateinische Bezeichnungen. Latein ist neben Englisch
womglich die einzige Weltsprache, die sich korrekt mit den ersten 128 ASCII-
Zeichen schreiben lt.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/


#ifndef MODULE_ROBOCOM_LINUXNQCH
#define MODULE_ROBOCOM_LINUXNQCH


#include "module_linuxnqc.h"
#include "module_robocom.h"
#include "module_comp_linuxnqc.h"
#include "module_conf.h"
#include <stdlib.h>
#include <string>
#include <sstream>
#include <fstream>


using namespace std;



///Robotersteuerungsklasse, welche die Konsolenschnittstelle des 
///Linux-NQC-Kompilierers verwendet.
///@param via_nqc Pfad zum NQC-Programm
///@param turris "-Susb" oder "" - USB- oder serieller IR-Turm.
///@param horreum_principalis Programmspeicherplatz (auf dem RCX) fr das
///Hauptprogramm
///@param confunditor Zugriff auf den Kompilierer
///@param lector Zugriff auf den Dateileser
//classis definita
struct module_robocom_linuxnqc :
  public module_robocom, public module_linuxnqc
{
  protected:
   string via_nqc; //via confunditoris

   string turris; //"-Susb" aut ""

   unsigned short horreum_principale;

   int status;

   module* confunditor; ///<Kompiliermodul
   module* lector; ///<Dateilesemodul
   
   //exflumen litterarum:
   ostringstream exflumen_lit; ///<Zur leichten Konvertierung int->string
   istringstream influmen_lit; ///<Zur leichten Konvertierung string->int
   
   //influmen datorum:
   fstream flumen_dat; ///<Zwischenspeicher fuer die nqc-Bildschirmausgabe

   virtual int mod_load();
   virtual int mod_unload();
   virtual int handle_msg(const char*);
   
   //functiones novae, heredium non sunt:

   bool proba_npos(string::size_type, char*);
   
   ///Hilfsfunktion zur Steuerung des Wechsels des 
   ///Roboterprogrammspeicherplatzes (nicht universell einsetzbar, da evtl.
   ///Programme auf dem Roboter gelscht werden.)
   bool permuta_horreum(string, bool renovatio = false); 


  public:
   module_robocom_linuxnqc();

   virtual int get_status();
   virtual char* get_name();

   virtual bool anima(); ///<Initialisiert den Roboter.
   virtual bool anima(float&); ///<Initialisiert den Roboter 
                               ///<und speichert seine Batteriespannung.
   virtual bool inanima(); ///<Schaltet den Roboter ab.
   virtual bool reanima(); ///<Reaktiviert den Roboter.

   virtual bool mitte_iussum(char*); ///<Sendet Befehl des gebotenen Namens.
   virtual bool mitte_iussum(string); ///<Sendet Befehl des gebotenen Namens.
   virtual bool mitte_iussum(int); ///<Sendet Befehl der gebotenen Nummer.

   ///Sendet ein Programm. (Schneiderleinbefehl, nicht Basisprogramm)
   //functiones novae, heredium non sunt.
   bool mitte_ordinem(string);
   bool mitte_ordinem(char*);
   ///Fhrt einen Schneiderleinbefehl aus.
   //functio nova, heredium non est.
   bool obsequere_ordini();

   ///Ping.
   virtual int ping();

   ///Empfangene Information zurckgeben/Informationen zuvor empfangen
   virtual char* accipe_nuntios();
   virtual string accipe_nuntios(bool);
};



#endif
