/*

~~~~~~~~~~~~~~~~
~ protocol.cpp ~
~~~~~~~~~~~~~~~~

Definitionen fr das Netzwerkprotkoll, das zur Kommunikation mit dem Roboter
verwendet wird. Weiterfhrende Informationen finden sich in der
Projektddokumentation.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/

#include <memory.h>
#include "protocol.h"

/// Erstellt eine Nachricht mit Standardparametern.
net_msg_hdr::net_msg_hdr( void )
{
	type = 0;
	cmd = 0;
	param_size = 0;
}

/// Erstellt eine Nachricht mit vorgegebenen Parametern.
///
/// @param type 	Der Typ der Nachricht, z.B. MSG_TYPE_SYTEM. Siehe protocol.h fr weitere Typen.
/// @param cmd 		Der Befehl der Nachricht, z.B. MSG_PING. Siehe protocol.h fr weitere
///		 	Befehle.
/// @param param_size 	Lnge der Parameter der Nachricht in Bytes
net_msg_hdr::net_msg_hdr( unsigned char type, unsigned char cmd, size_t param_size )
{
	this->type = type;
	this->cmd = cmd;
	this->param_size = param_size;
}

/// Setzt den Typ der Nachricht.
///
/// @param type Neuer Typ der Nachricht
/// @return Immer 0.
int net_msg_hdr::set_type( unsigned char type )
{
    this->type=type;
    return( 0 );
}

/// Setzt den Befehl der Nachricht.
///
/// @param cmd Neuer Befehl der Nachricht
/// @return Immer 0.
int net_msg_hdr::set_cmd( unsigned char cmd )
{
    this->cmd=cmd;
    return( 0 );
}

/// Setzt die Gre der Parameter der Nachricht
///
/// @param size Die neue Parametergre der Nachricht
/// @return Immer 0.
int net_msg_hdr::set_size( size_t size )
{
    this->param_size=size;
    return( 0 );
}

/// Gibt den Typ der Nachricht zurck.
///
/// @return Der Nachrichtentyp (0-255, z.B. MSG_TYPE_SYSTEM), siehe 
/// Definitionen in protocol.h
unsigned char net_msg_hdr::get_type( void ) const
{
	return( type );
}

/// Gibt den Befehl der Nachricht zurck.
///
/// @return Der Nachrichtenbefehl (0-255, z.B. MSG_PING), siehe Definitionen
/// in protocol.h.
unsigned char net_msg_hdr::get_cmd( void ) const
{
	return( cmd );
}

/// Gibt die Parametergre der Nachricht zurck.
///
/// @return Die Lnge des Parameterblocks in Bytes.
size_t net_msg_hdr::get_size( void ) const
{
	return( param_size );
}
