/*

~~~~~~~~~~~~~~
~ protocol.h ~
~~~~~~~~~~~~~~

Definitionen fr das Netzwerkprotkoll, das zur Kommunikation mit dem Roboter
verwendet wird. Weiterfhrende Informationen finden sich in der
Projektddokumentation.

Dieser Quelltext ist ursprnglich Teil des Programmes Eukalyptos, welches zur
Netzwerkfernsteuerung von Lego-RCX-2-Robotern dient und von Lutz Bch, Bastian
Rieck und Carlos Franke im Rahmen eines Praktikums am Interdisziplinren 
Zentrum fr Wissenschaftliches Rechnen (IWR) der Universitt Heidelberg 
geschaffen wurde.

*/

#ifndef PROTOCOL_H
#define PROTOCOL_H

#include <string>

using namespace std;

const unsigned char MSG_TYPE_SYSTEM = 1; 	///< Systemnachricht, z.B. Ping
const unsigned char MSG_TYPE_ROBOT = 2; 	///< Roboterbefehl, z.B. Vorwrts
const unsigned char MSG_PING = 1; 		///< Eine Ping-Nachricht, die nur dazu verwendet wird, 
						///< zu berprfen, ob die Verbindung noch besteht.
const unsigned char MSG_ANIMA = 2;		///< Diese Nachricht ndert den Status des Roboters und
						///< bertrgt das Roboterprogramm neu.
const unsigned char MSG_REANIMA = 3;		///< Reanimiert den Roboter					
const unsigned char MSG_INANIMA = 4;		///< Schaltet den Roboter aus
const unsigned char MSG_EXECUTION_TIME = 5;	///< Diese Nachricht gibt an, wie lange die Ausfhrung der Befehle,
						///< die der Nutzer eingegeben hat, voraussichtlich dauert.
const unsigned char MSG_EXECUTED_COMMANDS = 6;	///< Diese Nachricht enthlt als Parameter alle vom Roboter
						///< ausgefhrten Kommandos mit Parametern.
const unsigned char MSG_BATTERY_VOLTAGE = 7;	///< Schickt die aktuelle Batteriespannung, sodass der Nutzer immer wei,
						///< welche Genauigkeit er von dem Roboter zu erwarten hat.						
const unsigned char MSG_LOG = 8;		///< Enthlt Logbuchnachrichten der einzelnen Module, die fr den Serverteil
						///< des Programmes verwendet werden.
			 
const unsigned short CONVERSATION_READ_ARCHIVED_COMMANDS = 0;
const unsigned short CONVERSATION_READ_RECEIVED_COMMANDS = 1;
const unsigned short CONVERSATION_READ_EXECUTED_COMMANDS = 2;

/// Enthlt Headerdaten einer Nachricht, die ber das Netzwerk verschickt wird.
class net_msg_hdr
{
	public:
   	
		net_msg_hdr( void );	
		net_msg_hdr( unsigned char type, unsigned char cmd, size_t param_size = 0 );
		
        	int set_type( unsigned char type );
	        int set_cmd( unsigned char cmd );
        	int set_chksum( void );
	        int set_size( size_t size );
		
		unsigned char get_type( void ) const;
		unsigned char get_cmd( void ) const;
		
		size_t get_size( void ) const;

	private:


		unsigned char type; 	///< Kodiert den Typ der Nachricht (z.B. MSG_TYPE_SYSTEM)
		unsigned char cmd; 	///< Kodiert den Befehl der Nachricht (z.B. MSG_PING)
		size_t param_size; 	///< Gre des Parameterfeldes in Bytes
};

/// Speichert Headerdaten und Parameter einer Nachricht.
struct saved_net_msg
{
	net_msg_hdr header;
	string params;
};

#endif
