#include "sim_language.h"
#include <math.h>

#include <sstream>

const string OPS = "^/*-+";
const string whites("\r\n\t ");


/// Setzt die gegebenen Werte fr die Variablen ein und errechnet
/// damit rekursiv den aktuellen Wert des gesamten Terms
///
/// @param params Der Vektor, der die Werte fr die Variablen p_0, p_1,... enthlt (unsinged char)
///
/// @return Der gerundete, aktuelle Wert des Terms (unsigned char)
unsigned long term::evaluate( vector<unsigned char> params )
{
 	int atemp=0;
 	int btemp=0;
 	if ( atype==NUMBER ) { atemp=avalue; }
 	if ( btype==NUMBER ) { btemp=bvalue; }
 	if ( atype==VARIABLE ) { atemp=params[avalue]; }
 	if ( btype==VARIABLE ) { btemp=params[bvalue]; }
	// Der rekursive Aufruf terminiert, da nicht jeder Term einen Term enthlt.
	// Das liegt daran, dass die eingegebenen Formeln auch nur endlich sein knnen.
 	if ( atype==TERM ) { atemp=aterm[0].evaluate(params); }
 	if ( btype==TERM ) { btemp=bterm[0].evaluate(params); }
 	switch (op)
    {
	 	   case ADD: { return(atemp+btemp); }
         case SUBT: { return(atemp-btemp); }
         case MULT: { return(atemp*btemp); }
         case DIV: { return(static_cast<int>( round(atemp/static_cast<double>(btemp) ))); }
         case POW: { return(static_cast<int>( round(pow((float)atemp,(float)btemp)))); }
    }
    return 0;
}

/// Errechnet genau wie evaluate den aktuellen Wert eines Terms,
/// allerdings mit hherer Genauigkeit
///
/// @param params Der Vektor, der die Werte fr die Variablen p_0, p_1,... enthlt (double)
///
/// @return Der aktuelle Wert des Terms (double)
double term::precisely_evaluate( vector<double> params )
{
   double atemp=0;
   double btemp=0;
   if ( atype==NUMBER ) { atemp=avalue; }
   if ( btype==NUMBER ) { btemp=bvalue; }
   if ( atype==VARIABLE ) { atemp=params[(int)avalue]; }
   if ( btype==VARIABLE ) { btemp=params[(int)bvalue]; }
	// Der rekursive Aufruf terminiert, da nicht jeder Term einen Term enthlt.
	// Das liegt daran, dass die eingegebenen Formeln auch nur endlich sein können.
   if ( atype==TERM ) { atemp=aterm[0].precisely_evaluate(params); }
   if ( btype==TERM ) { btemp=bterm[0].precisely_evaluate(params); }
   switch (op)
    {
         case ADD: { return(atemp+btemp); }
         case SUBT: { return(atemp-btemp); }
         case MULT: { return(atemp*btemp); }
         case DIV: { return ( atemp/static_cast<double>(btemp) ); }
         case POW: { return ( pow(atemp,btemp)); }
    }
    return 0;
}

/// Ntzlich zum Debuggen: Gibt die Struktur eines Terms in einer Formel
/// wieder, damit man sieht, ob term_terminate korrekt umgewandelt hat.
/// Ist also die "Umkehrfunktion" von term_terminate
string term::strip()
{
   string tmpstr="";
   ostringstream conv;
   
   if ( atype==VARIABLE ) { tmpstr.append("p_"); }
   if ( atype==VARIABLE || atype==NUMBER )
   { 
      conv << static_cast<int>( avalue );
      tmpstr.append( conv.str() );
   }
   if ( atype==TERM ) { tmpstr.append("("+aterm[0].strip()+")"); }
   switch (op)
    {
         case ADD: { tmpstr.append("+"); break; }
         case SUBT: { tmpstr.append("-"); break; }
         case MULT: { tmpstr.append("*"); break; }
         case DIV: { tmpstr.append("/"); break; }
         case POW: { tmpstr.append("^"); break; }
    }
   if ( btype==VARIABLE ) { tmpstr.append("p_"); }
   if ( btype==VARIABLE || btype==NUMBER )
   { 
      conv.clear();
      conv.str("");
      conv << static_cast<int>( bvalue );
      tmpstr.append( conv.str() ); 
   }
   if ( btype==TERM ) { tmpstr.append("("+bterm[0].strip()+")"); }
   return tmpstr;
}

/// Bereitet die Formel zur Umwandlung in einen Term vor, indem
/// es Klammern setzt. So bekommt die Funktion term_terminate
/// in jedem Fall in eindeutiger Form.
/// Diese Funktion stellt auch die "Potenz-vor-Punkt-vor-Strichrechung" sicher.
///
/// @param input Die Formel, die es vorzubereiten gilt
///
/// @return Die Formel in einer fr term_terminate schneren Form
string term_set_parenthesis(string input)
{
 	 //Weirume entfernen:
	 while ( input.find_first_of("\n\t ") != string::npos )
	 { input.erase(input.find_first_of("\n\t "),1); }
	 //Geteilt-Zeichen vereinheitlichen:
	 while ( input.find_first_of(":") != string::npos )
	 { input.replace(input.find_first_of(":"),1,"/"); }
	 //Verhindern, dass kein Rechenzeichen vorkommt:
	 if ( input.find_first_of(OPS) == string::npos )
	 { input.append("+0"); }
    
	 for (int c=0; c<OPS.length(); c++) //Alle Rechenzeichen durchgehen
	 {
	  	 int ccount=0;
	  	 int i;
	  	 for (i=0; i<input.length(); i++) //Exemplare dieses RZ zhlen
	  	 { if (input[i]==OPS[c]) {++ccount;} }
	  	 int pcount;
	  	 for (i=1; i<=ccount; i++)  //Alle Exemplare dieses RZ durchgehen
	  	 {
		  	 int ccup=i;
			 int j=0;
			 int ccpos=0;
			 while (ccup!=0)
			 {
			  	   if (input[j]==OPS[c]) {--ccup;}
			  	   if (ccup==0) {ccpos=j;}
			  	   ++j;
			 }
			 if (input[ccpos+1]=='(')
			 {
			  	j=ccpos+1;
			  	pcount=1;
		      while (pcount!=0)
			  	{ ++j;
		          if (input[j]=='(') {++pcount;}
		          if (input[j]==')') {--pcount;}
		        }
              ++j;
		     }
		     else
			 {
			  	 j=ccpos+1;             
  		  	 while ( (((string)"0123456789p_").find_first_of(input[j])!= string::npos) && (j<input.length()) )
			  	 { ++j; }
			 }	 
			 input.insert(j,")"); //j steht rechts vom B-Ausdruck
          
			 if (input[ccpos-1]==')')
			 {
				j=ccpos-1;
				pcount=1;
				while (pcount!=0 && j>1)
				  { --j;
				     if (input[j]=='(') {--pcount;}
				     if (input[j]==')') {++pcount;}
              }
		     }
		     else
		     {
			  	 j=ccpos-1;
			  	 while( ((string)"0123456789p_").find_first_of(input[j])!= string::npos && j>=0 )
			  	 { --j; }
             ++j;
			 }
			 input.insert(j,"("); //j steht links vom A-Ausdruck
         }
	 }
	 return input;
}

/// Verwandelt einen Formel, also einen String wie "(p_0+5)*5+p_4*3/(1/2)"
/// in einen quivalenten Term. Dazu wird zunächst set_parenthesis aufgerufen,
/// um den String vorzubereiten
///
/// @param input Die Formel in Rohform
///
/// @return Der fertige Term. Es ist ein Vektor, weil man scheinbar die
///         selbst-definierte Klasse Term nicht zurckgeben kann.
///         Der Term befindet sich an 0-ter Stelle des zurckgegebenen Vektors
vector<term> term_terminate(string input)
{
    vector<term> output;
 	 term tmpterm;
 	 bool outerp=true;
 	 int pcount=0;
 	 int i=0;
 	 int j=0;
    
    //Vorbereiten:
    input=term_set_parenthesis(input);
 	 
 	 //uere Klammern entfernen:
 	 while (outerp && input.find_first_of("(")==0)
 	 {
	  	   outerp=false;
	  	   pcount=0;
	  	   i=0;
	  	   while ((pcount>=0) || (i==(input.length()-1)))
	  	   {
		   		 ++i;
		   		 if (input[i]=='(') { ++pcount; }
		   		 if (input[i]==')') { --pcount; }
		   }
		   if (i==input.length())
		   {
				 input.erase(input.length()-1,1);
				 input.erase(0,1);
				 outerp=true;
		   }
     }  

     //Finde Hauptrechenzeichen (das mit minimaler Umklammerung)
	 i=0;
	 pcount=0;
	 if (input[i]=='(') { ++pcount; }
	 while ( (input[i]!='+' && input[i]!='-' && input[i]!='*' && input[i]!='/' && input[i]!='^') || (pcount!=0) )
	 {
	  	   ++i;
	  	   if (input[i]=='(') { ++pcount; }
	  	   if (input[i]==')') { --pcount; }
     }
     int cpos=i;
     
     tmpterm.op=OPS.find_first_of(input[cpos]);
     i=cpos-1;
	  if (input[i]==')')
     {
        tmpterm.atype=TERM;
        pcount=1;
        i=cpos-2;
        while (pcount>0)
        {
              if (input[i]==')') { ++pcount; }
              if (input[i]=='(') { --pcount; }
              --i;
        }
        ++i;
        tmpterm.aterm=term_terminate(input.substr(i,cpos-i));
     }
     else
     {
         j=i;
		   bool tmpbool=true;
		   while (tmpbool && input[j]!='(' && j>0)
		   {
		        --j;
                tmpbool=input[j]!='_';
		   }
		   if ( input[j]=='(' || input[j]=='_' ) { ++j; }
		   if (tmpbool)
		   { tmpterm.atype=NUMBER; }
		   else
		   { tmpterm.atype=VARIABLE; }
        if (tmpterm.atype==NUMBER || tmpterm.atype==VARIABLE)
        { tmpterm.avalue=atoi(input.substr(j, cpos-j).c_str()); }
     }
      
	 i=cpos+1;
	 if (((string)"0123456789").find_first_of(input[i])!=string::npos) {tmpterm.btype=NUMBER;}
	 if (input[i]=='p') {tmpterm.btype=VARIABLE;}
	 if (input[i]=='(') {tmpterm.btype=TERM;}
	 
	 ++cpos;
	 if (tmpterm.btype==NUMBER)
	 {
        while (((string)"0123456789").find_first_of(input[i])!=string::npos) {++i;}
        tmpterm.bvalue=atoi(input.substr(cpos,i-cpos).c_str());
     }
     if (tmpterm.btype==VARIABLE)
     {
	    i=i+2;
	    while (((string)"0123456789").find_first_of(input[i])!=string::npos) {++i;}
	    tmpterm.bvalue=atoi(input.substr(cpos+2,i-cpos-2).c_str());
     }
     if (tmpterm.btype==TERM)
     {
	    pcount=1;
	    i=cpos+1;
	    while (pcount>0)
	    {
		 	  if (input[i]=='(') {++pcount;}
		 	  if (input[i]==')') {--pcount;}
		 	  ++i;
        }
        tmpterm.bterm=term_terminate(input.substr(cpos,i-cpos));
	 }
	 output.push_back(tmpterm);
	 return output;
}

/// Verwandelt einen String mit Elementarbefehlen, die als Parameter
/// Formeln tragen ("Vorwaerts(p_0*10)"), in einen Vektor aus conf_command
/// um, d.h. in einen Vektor aus Paaren von Bezeichnern ("Vorwaerts") und Termen.
///
/// @param input Der String aus Befehlen
///
/// @return Ein Vektor von conf_command, also Namen von Befehlen und ihr Parameter in Term-Form
vector<conf_command> term_parse_cmd(string input)
{
       vector<conf_command> output;
       conf_command tmp_cmd;
       
       //Weirume entfernen:
       unsigned int end;
       while (input.find_first_of(whites) != string::npos)
       {input.erase(input.find_first_of(whites),1);}
       
       while (input.size()!=0)
       {
               //##### Bezeichner-Modus #####//
               //Bezeichner extrahieren: 
               end=input.find_first_of('(');
               string name=input.substr(0,end);
               input.erase(0,end+1);
                              
               transform(name.begin(), name.end(), name.begin(), (int(*)(int)) toupper);
               tmp_cmd.name=name;
                           
               //##### Parameter-Modus #####//
               //Eventuelle innere Klammern bergehen
               end=0;
               unsigned int ps=1; //Klammern-Zhler
               while (ps!=0 && end<input.length())
               {
                     switch (input[end])
                     {
                              case '(': {++ps; break;}
                              case ')': {--ps; break;}
                     }
                     ++end;
               }
               tmp_cmd.formula=term_terminate(input.substr(0,end-1))[0];
               input.erase(0,end);
               output.push_back(tmp_cmd);      
       }  
       return( output );    
}
